/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.client_attribute.grpc;

import de.ozgcloud.apilib.client_attribute.OzgCloudBooleanClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.client_attribute.OzgCloudDoubleClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudLongClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudStringClientAttribute;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class OzgCloudClientAttributeGrpcService implements OzgCloudClientAttributeService {

	private final ClientAttributeServiceBlockingStub serviceBlockingStub;
	private final OzgCloudClientAttributeMapper clientAttributeMapper;
	private final OzgCloudCallContextProvider contextProvider;

	@Override
	public void setClientAttribute(OzgCloudBooleanClientAttribute clientAttribute, OzgCloudVorgangId vorgangId) {
		doSetClientAttribute(clientAttributeMapper.toGrpc(clientAttribute), vorgangId);
	}

	@Override
	public void setClientAttribute(OzgCloudStringClientAttribute clientAttribute, OzgCloudVorgangId vorgangId) {
		doSetClientAttribute(clientAttributeMapper.toGrpc(clientAttribute), vorgangId);
	}

	@Override
	public void setClientAttribute(OzgCloudLongClientAttribute clientAttribute, OzgCloudVorgangId vorgangId) {
		doSetClientAttribute(clientAttributeMapper.toGrpc(clientAttribute), vorgangId);
	}

	@Override
	public void setClientAttribute(OzgCloudDoubleClientAttribute clientAttribute, OzgCloudVorgangId vorgangId) {
		doSetClientAttribute(clientAttributeMapper.toGrpc(clientAttribute), vorgangId);
	}

	void doSetClientAttribute(GrpcClientAttribute clientAttribute, OzgCloudVorgangId vorgangId) {
		getAttributesServiceStub().set(buildRequest(clientAttribute, vorgangId.toString()));
	}

	GrpcSetClientAttributeRequest buildRequest(GrpcClientAttribute attribute, String vorgangId) {
		return GrpcSetClientAttributeRequest.newBuilder().setVorgangId(vorgangId).setAttribute(attribute).build();
	}

	ClientAttributeServiceBlockingStub getAttributesServiceStub() {
		return serviceBlockingStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}

}
