/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.command.grpc;

import java.util.Objects;
import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;
import de.ozgcloud.common.datatype.StringBasedValue;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCreateCommandRequest;

@Mapper(uses = GrpcObjectMapper.class)
public interface OzgCloudCommandMapper {

	@Mapping(target = "redirectRequestOrBuilder", ignore = true)
	@Mapping(target = "orderStringBytes", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "callContextOrBuilder", ignore = true)
	@Mapping(target = "bodyObjOrBuilder", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBodyObj", ignore = true)
	@Mapping(target = "mergeCallContext", ignore = true)
	@Mapping(target = "mergeRedirectRequest", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "removeBody", ignore = true)
	@Mapping(target = "callContext", ignore = true)
	@Mapping(target = "redirectRequest", ignore = true)
	@Mapping(target = "relationIdBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "vorgangIdBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "bodyBuilderList", ignore = true)
	@Mapping(target = "bodyList", ignore = true)
	@Mapping(target = "bodyOrBuilderList", ignore = true)

	@Mapping(target = "bodyObj", source = "bodyObject")
	@Mapping(target = "orderString", source = "order")
	GrpcCreateCommandRequest toCreateRequest(OzgCloudCommand command);

	default String toString(StringBasedValue stringBasedValue) {
		return Optional.ofNullable(stringBasedValue).map(Objects::toString).orElse(null);
	}

	default String map(OzgCloudVorgangId value) {
		return Optional.ofNullable(value).map(Objects::toString).orElse(null);
	}
}
