/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.client.autoconfigure;

import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.test.annotation.DirtiesContext;

import de.ozgcloud.apilib.autoconfig.ApiLibTestApplication;
import de.ozgcloud.apilib.user.GrpcOzgCloudUserProfileService;
import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.apilib.user.OzgCloudUserProfile;
import de.ozgcloud.apilib.user.OzgCloudUserProfileService;

public class OzgCloudUserProfileServiceStarterITCase {

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class)
	@DirtiesContext()
	class TestNoOzgCloudUserProfileService {

		@Autowired
		private Optional<OzgCloudUserProfileService> ozgCloudUserProfileService;

		@Test
		void shouldStart() {
			assertThat(ozgCloudUserProfileService).isEmpty();
		}
	}

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class, properties = {
		"ozgcloud.user-manager.address=address"
	})
	@DirtiesContext
	class TestWithOzgCloudUserProfileService {

		@Autowired
		private OzgCloudUserProfileService ozgCloudUserProfileService;

		@Test
		void shouldStart() {
			assertThat(ozgCloudUserProfileService).isInstanceOf(GrpcOzgCloudUserProfileService.class);
		}
	}

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class, properties = {
		"ozgcloud.file-manager.address=address"
	})
	@Import(TestWithExistingOzgCloudFileService.Config.class)
	class TestWithExistingOzgCloudFileService {

		@Autowired
		private OzgCloudUserProfileService ozgCloudUserProfileService;

		@Test
		void shouldStart() { // NOSONAR
			assertThat(ozgCloudUserProfileService).isInstanceOf(DummyOzgCloudUserProfileService.class);
		}

		record DummyOzgCloudUserProfileService() implements OzgCloudUserProfileService {

			@Override
			public OzgCloudUserProfile getById(final OzgCloudUserId userId) {
				return OzgCloudUserProfile.builder().build();
			}
		}

		static class Config {

			@Bean
			OzgCloudUserProfileService zgCloudUserProfileService() {
				return new DummyOzgCloudUserProfileService();
			}
		}
	}
}
